"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs2/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _values = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/values"));

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/json/stringify"));

var _promise = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/promise"));

var _embarkUtils = require("embark-utils");

var path = _interopRequireWildcard(require("path"));

var _contractEnhanced = require("./contractEnhanced");

var _path2 = require("./path");

class Coverage {
  constructor(embark, options) {
    this.embark = embark;
    this.contracts = void 0;
    this.deployedContracts = [];
    this.web3Contracts = [];
    this.originalContractFiles = [];
    this.fs = void 0;
    this.fs = embark.fs;
    this.fs.ensureDirSync((0, _path2.coverageContractsPath)());
    this.originalContractFiles = embark.config.contractsFiles;
    this.contracts = this.getContracts();
    this.embark.events.setCommandHandler("coverage:prepareContracts", async done => {
      await this.prepareContracts();
      this.swapContracts();
      done();
    });
    this.embark.events.on("tests:ready", this.pushDeployedContracts.bind(this));
    this.embark.events.on("tests:finished", this.produceCoverageReport.bind(this));
    this.embark.events.on("tests:manualDeploy", this.registerWeb3Contract.bind(this));
  }

  getContracts() {
    const solcVersion = this.embark.config.embarkConfig.versions.solc;
    return this.originalContractFiles.map(file => new _contractEnhanced.ContractEnhanced(file.path, solcVersion));
  }

  async prepareContracts() {
    const promises = this.contracts.map(async contract => {
      contract.instrument();
      contract.save();
    });
    await _promise.default.all(promises);
  }

  swapContracts() {
    this.embark.config.embarkConfig.contracts = this.originalContractFiles.map(file => path.join((0, _path2.coverageContractsPath)(), file.path));
    this.embark.config.contractsFiles = [];
    this.embark.config.reloadConfig();
  }

  async pushDeployedContracts() {
    const newContracts = await this.getDeployedContracts();
    this.deployedContracts = this.deployedContracts.concat(newContracts);
  }

  async produceCoverageReport(cb) {
    const web3Contracts = await this.getWeb3Contracts();
    await _promise.default.all(this.collectEvents(web3Contracts));
    this.writeCoverageReport(cb);
  }

  writeCoverageReport(cb) {
    this.fs.ensureDirSync(path.join((0, _embarkUtils.dappPath)(), ".embark"));
    const coveragePath = path.join((0, _embarkUtils.dappPath)(), ".embark", "coverage.json");
    const coverageReport = this.contracts.reduce((acc, contract) => {
      if (contract.source) {
        acc[contract.filepath] = contract.coverage;
      }

      return acc;
    }, {});
    this.fs.writeFile(coveragePath, (0, _stringify.default)(coverageReport, null, 2), cb);
  }

  collectEvents(web3Contracts) {
    return web3Contracts.map(async web3Contract => {
      const events = await web3Contract.getPastEvents("allEvents", {
        fromBlock: 0
      });
      this.contracts.forEach(contract => contract.updateCoverage(events));
    });
  }

  getWeb3Contract(deployedContract) {
    return new _promise.default(resolve => {
      const address = deployedContract.deployedAddress;
      const abi = deployedContract.abiDefinition;
      this.embark.events.request("blockchain:contract:create", {
        address,
        abi
      }, web3Contract => {
        resolve(web3Contract);
      });
    });
  }

  getDeployedContracts() {
    return new _promise.default((resolve, reject) => {
      this.embark.events.request("contracts:all", (error, contracts) => {
        if (error) {
          return reject(error);
        }

        resolve((0, _values.default)(contracts));
      });
    });
  }

  async getWeb3Contracts() {
    const web3Contracts = this.deployedContracts.filter(deployedContract => deployedContract.deployedAddress).map(deployedContract => this.getWeb3Contract(deployedContract));
    return (await _promise.default.all(web3Contracts)).concat(this.web3Contracts);
  }

  registerWeb3Contract(web3Contract) {
    this.web3Contracts.push(web3Contract);
  }

}

exports.default = Coverage;
//# sourceMappingURL=index.js.map